

function [T,Y]=Adamsbf(t0,tmax,n,y_inn,f)

%___________________________________________________________________________________
%
%  Funzione Adamsbf
%  ________________
%
%  Chiamata: [T,Y]=Adamsbf(t0,tmax,n,y_inn,f);   
%
%
% La function Adamsbf utilizza il metodo di Adams-Bashforth  
% del quarto ordine esplicito per determinare i valori approssimati
% della soluzione di un problema di Cauchy,  nei nodi t(i),
% i=1,2,...,n+1 contenuti in  un intervallo [t0,tmax].
% Il problema (scalare o vettoriale) e' del tipo:
% 
% 
% y'(t)=f(t,y(t))     con t in (t0,tmax]
% y(t0)=yo
%
%   
% dove:
% y(t)=(y1(t),y2(t),...,ym(t))
% f(t,y(t))=(f1(t,y(t)),f2(t,y(t)),...,fm(t,y(t))
% y(t0)=(y1(t0),y2(t0),...,ym(t0))=(yo1,yo2,...,yom)=yo
%
% con m=1 nel caso scalare e m>1 nel caso vettoriale.
%___________________________________________________________________________ 
%
%
% Parametri di input della function:
% ___________________________________
%
% t0,tmax  estremi dell'intervallo di integrazione.
%
%  n       numero di sottointervalli [t(i),t(i+1)], i=1,...,n
%          in cui si desidera suddividere l'intervallo [t0,tmax].    
%
% y_inn    matrice di dimensione 4 x m la cui i-esima riga di 
%          lunghezza m contiene l'approssimazione ottenuta, ad 
%          esempio, dal metodo di Runge-Kutta del quarto ordine, 
%          della soluzione calcolata nel nodo t(i): 
%          y(ti)=[y1(ti),y2(ti),...,ym(ti)], i=0,1,2,3.
% 
% f        vettore colonna di dimensione m  in cui ogni componente e' 
%          una stringa contenente le espressioni f1, f2,..,fm scritte in
%          funzione delle variabili t e y; si puo' creare il vettore f 
%          utilizzando il comando 'strvcat', nel seguente modo: 
%          f=strvcat(f1,f2,...,fm). 
%
%
% Parametri di output della function:
% ___________________________________
%
% T        vettore colonna di dimensione n+1 contenente i nodi t0=t(1),
%          t(2),...,t(n),t(n+1)=tmax.
%
% Y        matrice di dimensione (n+1)x m la cui i-esima riga di lunghezza 
%          m contiene l'approssimazione della funzione y(t)=[y1(t),...,ym(t)], 
%          calcolata nel nodo t(i): y(t(i))=[y1(t(i)),y2(t(i)),...,ym(t(i))].
% 
%___________________________________________________________________________
%
%
% 1° esempio  di utilizzo della function  applicata al seguente 
% problema di Cauchy (caso scalare con m=1):
%
% y'(t)=-y+2cos(t)    t e' in (0,pi]
% y(0)=1
%
% con soluzione:  y(t)=sin(t)+cos(t).
%
%
%      clear all
%      t0=0; tmax=pi;
%      y0=1;
%      n=300;
%      f='-y+2*cos(t)';
%      h=(tmax-t0)/n;
%
%      [t_inn,y_inn]= Rungekutta4(t0,t0+3*h,3,y0,f);
%      [T,Y]=Adamsbf(t0,tmax,n,y_inn,f);
%       
%___________________________________________________________________________
%
%
% 2° esempio  di utilizzo della function  applicata al seguente 
% problema di Cauchy (da ricondursi a un problema vettoriale del primo 
% ordine con m=2):
%
% 
% z''(t)=(32+2t^3-z(t)z'(t))/8    t e' in (1,3]   (*)
% z(1)=17
% z'(1)=-14.
%
% Il problema di Cauchy (*) e' del secondo ordine; per poter applicare
% la function Adamsbf a tale problema e' necessario trasformare (*)
% in un problema del 1° ordine. 
% Ponendo z(t)=y1(t), il problema (*) si trasforma nel problema 
% equivalente:
%
% y1'(t)=y2(t)                    t e' in (1,3]   (**)
% y2'(t)=(32+2t^3-y1(t)y2(t))/8
% y1(1)=17
% y2(1)=-14.
%
% La soluzione vera di (**) e':
% y1(t)=t^2+16/t
% y2(t)=2t-16/t^2.
%
% La soluzione vera di (*) e' allora:
% z(t)=y1(t)=t^2+16/t.
% La y2(t) corrisponde alla derivata z'(t).
% 
%___________________________________________________________________________
%
%
% clear all
% t0=1; tmax=3;
% y0=[17 -14];
% n=200;
% f1='y(2)';
% f2='(32+2*t.^3-y(1)*y(2))./8';
% f=strvcat(f1,f2);
% h=(tmax-t0)/n;
%
% [t_inn,y_inn]= Rungekutta4(t0,t0+3*h,3,y0,f);
% [T,Y]=Adamsbf(t0,tmax,n,y_inn,f);
%       
% % Osservare che in questo caso Y e' una matrice di dimensione 
% % n+1 x 2 contenente nella prima colonna i valori y1(t(i)), 
% % i=1,2,...,n+1, che sono  le approssimazioni nei nodi della 
% % soluzione del problema (*); la seconda colonna di Y contiene
% % i valori y2(t(i)) che sono le approssimazioni della derivata
% % prima della soluzione di (*).
%
%___________________________________________________________________________






h=(tmax-t0)/n;
[l,m]=size(y_inn);      
Y=y_inn;
T=[t0:h:t0+n*h]';
r=4;

for i=1:r
  t=T(i); y=Y(i,:);
  for j=1:m
      fun(i,j)=eval(f(j,:));         
  end
end  
coef1=[-9 37 -59 55 ]/24;
for i=r:n
    Y(i+1,:)=Y(i,:)+h*(coef1*fun);   
    
    t=T(i+1); y=Y(i+1,:);            
    for j=1:m
   	   fun(r+1,j)=eval(f(j,:));
    end
    fun=[fun(2,:);fun(3,:);fun(4,:);fun(5,:)];
end




